﻿using System.Collections;
using System.Windows.Controls;
using System.Windows;
using System.Linq;
using System;
using System.Collections.Generic;
using System.Windows.Media;
using System.Windows.Input;

namespace SampleGame
{
    public class GameEngine
    {
        Grid gameGrid;
        Canvas gameCanvas;
        int currDrawIndex;

        List<Image> listImage;
        int currDrawIndex_Image;
        List<TextBlock> listTextBlock;
        int currDrawIndex_TextBlock;

        public Random Rand;

        public Point pointMouseDown;
        public bool MouseDownEvent;
        public Point pointMouseUp;
        public bool MouseUpEvent;
        public Point pointMouse;

        public GameEngine(UserControl Page, Grid grid, Canvas canvas)
        {
            gameGrid = grid;
            gameCanvas = canvas;
            gameCanvas.Children.Clear();
            currDrawIndex = 0;

            Rand = new Random(Environment.TickCount);

            listImage = new List<Image>();
            listTextBlock = new List<TextBlock>();

            Page.MouseLeftButtonDown += new MouseButtonEventHandler(Page_MouseLeftButtonDown);
            Page.MouseLeftButtonUp += new MouseButtonEventHandler(Page_MouseLeftButtonUp);
            Page.MouseMove += new MouseEventHandler(Page_MouseMove);
            Page.MouseWheel += new MouseWheelEventHandler(Page_MouseWheel);
            Page.KeyDown += new KeyEventHandler(Page_KeyDown);
            Page.KeyUp += new KeyEventHandler(Page_KeyUp);
        }

        #region 드로잉 처리
        public void DrawBegin()
        {
            foreach(object img in gameCanvas.Children)
            {
                if(img.GetType().Name.CompareTo("Image") == 0)
                {
                    ((Image)img).Visibility = System.Windows.Visibility.Collapsed;
                }
                else if (img.GetType().Name.CompareTo("TextBlock") == 0)
                {
                    ((TextBlock)img).Visibility = System.Windows.Visibility.Collapsed;
                }
            }

            currDrawIndex = 0;
            currDrawIndex_Image = 0;
            currDrawIndex_TextBlock = 0;
        }

        public void DrawEnd()
        {

        }

        private Image GetImageObject()
        {
            Image image;
            if (currDrawIndex_Image >= listImage.Count)
            {
                image = new Image();
                listImage.Add(image);
                gameCanvas.Children.Add(image);
            }
            else
            {
                image = listImage[currDrawIndex_Image];
            }

            return image;
        }
        private TextBlock GetTextBlockObject()
        {
            TextBlock textBlock;

            if (currDrawIndex >= listTextBlock.Count)
            {
                textBlock = new TextBlock();
                listTextBlock.Add(textBlock);
                gameCanvas.Children.Add(textBlock);
            }
            else
            {
                textBlock = listTextBlock[currDrawIndex_TextBlock];
            }

            return textBlock;
        }

        public void DrawImage(Image img, double x, double y)
        {
            Image image;

            image = GetImageObject();

            image.Source = img.Source;
            image.SetValue(Canvas.LeftProperty, x);
            image.SetValue(Canvas.TopProperty, y);

            image.SetValue(Canvas.ZIndexProperty, currDrawIndex);

            image.Visibility = Visibility.Visible;

            currDrawIndex++;
            currDrawIndex_Image++;
        }
        public void DrawText(string text, string font, Color c, int size, double x, double y)
        {
            TextBlock textBlock;

            textBlock = GetTextBlockObject();

            textBlock.Text = text;
            textBlock.FontFamily = new FontFamily(font);
            textBlock.FontSize = size;
            textBlock.Foreground = new SolidColorBrush(c);

            textBlock.SetValue(Canvas.LeftProperty, x);
            textBlock.SetValue(Canvas.TopProperty, y);

            textBlock.SetValue(Canvas.ZIndexProperty, currDrawIndex);

            textBlock.Visibility = Visibility.Visible;

            currDrawIndex++;
            currDrawIndex_TextBlock++;
        }
        #endregion

        #region 사운드처리
        public void PlaySound(MediaElement media)
        {
            MediaElement sound = new MediaElement();
            sound.Source = media.Source;
            sound.MediaEnded += new RoutedEventHandler(sound_MediaEnded);
            gameGrid.Children.Add(sound);
        }

        void sound_MediaEnded(object sender, RoutedEventArgs e)
        {
            MediaElement sound = sender as MediaElement;
            if (sound == null)
                return;
            gameGrid.Children.Remove(sound);
        }
        #endregion

        #region 입력 처리
        void Page_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            pointMouseDown = e.GetPosition(null);
            MouseDownEvent = true;
        }
        void Page_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            // 마우스 뗌
        }
        void Page_MouseMove(object sender, MouseEventArgs e)
        {
            pointMouse = e.GetPosition(null);
        }
        void Page_MouseWheel(object sender, MouseWheelEventArgs e)
        {
            // 마우스 휠 움직임
        }
        void Page_KeyDown(object sender, KeyEventArgs e)
        {
            if(e.Key != Key.Unknown)
            {
                // 키 누름
            }
        }
        void Page_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.Key != Key.Unknown)
            {
                // 키 뗌
            }
        }
        #endregion
    }
}