﻿using System.Collections.Generic;
using System.Windows;
using System.Windows.Media;
using System.Windows.Browser;

namespace SampleGame
{
    public enum BlockType
    {
        None,
        Red,
        Green,
        Blue,
        Yellow,
        Purple,
        Max,
    }
    public class Block
    {
        public BlockType blockType;
        public double x, y;
    }

    public class GameProcess
    {
        Block[,] blocks;
        Block dropBlock;
        int idxDrop;
        double dropSpeed;

        int score;
        int highScore;

        int selectX, selectY;
        bool isSelect;

        const int BLOCK_WIDTH = 64;
        const int BLOCK_HEIGHT = 32;
        const int GAME_WIDTH = 320;
        const int GAME_HEIGHT = 450;

        const int MAX_INDEX_Y = 13;
        const int MAX_INDEX_X = 4;

        bool isPlaying;
        /// <summary>
        /// score
        /// </summary>
        public static ScriptObject hScore = null;
        public static ScriptObject owner = null;
        public static string ownerData = null;

        public GameProcess()
        {

        }

        public void Init() // 초기화
        {
            CRes.LoadRes(RES.GameRes);
            blocks = new Block[MAX_INDEX_Y + 1, MAX_INDEX_X + 1];
            isPlaying = false;
        }

        public void Terminate() // 종료
        {
            CRes.ReleaseRes(RES.GameRes);
        }

        public void InitGame()
        {
            for (int i = 0; i <= MAX_INDEX_Y; i++)
            {
                for(int j=0; j<=MAX_INDEX_X; j++)
                {
                    blocks[i, j] = null;
                }
            }

            dropBlock = null;

            dropSpeed = 5.0;
            score = 0;
            idxDrop = 0;
            isSelect = false;
        }

        public void Pulse() // 연산 처리
        {
            MouseProcess();

            if (isPlaying == false) return;
            
            #region 떨어지는 블록 처리
            if (dropBlock == null)
            {
                dropBlock = new Block();
                dropBlock.blockType = (BlockType)MainPage.gameEngine.Rand.Next((int)BlockType.None + 1, (int)BlockType.Max);
                dropBlock.x = idxDrop * BLOCK_WIDTH + BLOCK_WIDTH/2;
                dropBlock.y = -BLOCK_HEIGHT/2;
            }

            double nextY = (dropBlock.y + BLOCK_HEIGHT / 2 + dropSpeed);
            int nextYIdx = (int)(GAME_HEIGHT - nextY)/BLOCK_HEIGHT;

            if(nextY > GAME_HEIGHT) // 바닥에 도착하면
            {
                dropBlock.y = GAME_HEIGHT - BLOCK_HEIGHT/2;
                blocks[0, idxDrop] = dropBlock;

                dropBlock = null;
                idxDrop = (idxDrop + 1) % 5;
                if (dropSpeed < 30)
                    dropSpeed += 0.05;
            }
            else if (blocks[nextYIdx, idxDrop] != null)
            {
                if (nextYIdx+1 > MAX_INDEX_Y) // 게임오버
                {
                    ////////////////
                    if (hScore == null)
                    {
                        owner = HtmlPage.Window.Eval("document.getElementById(\"owner_info\")") as ScriptObject;
                        ownerData = owner.GetProperty("ownerData") as string;
                        string[] info = ownerData.Split('#');
                        owner = HtmlPage.Window.Eval("document.getElementById(\"userid\")") as ScriptObject;
                        owner.SetProperty("value", info[0]);

                        hScore = HtmlPage.Window.Eval("document.getElementById(\"score\")") as ScriptObject;
                        if (hScore != null)
                        {
                            hScore.SetProperty("value", highScore);
                            hScore = HtmlPage.Window.Eval("document.getElementById(\"button\")") as ScriptObject;
                            hScore.Invoke("click");
                        }
                    }
                    /////////////////
                    isPlaying = false;
                    return;
                }
                dropBlock.y = GAME_HEIGHT - (nextYIdx + 1) * BLOCK_HEIGHT - BLOCK_HEIGHT/2;
                blocks[nextYIdx+1, idxDrop] = dropBlock;

                dropBlock = null;
                idxDrop = (idxDrop + 1) % 5;
                if (dropSpeed < 30)
                    dropSpeed += 0.05;
            }
            else
            {
                dropBlock.y += dropSpeed;
            }
            #endregion

            #region 블록 없애기
            bool[,] destroyCheck = new bool[MAX_INDEX_Y + 1, MAX_INDEX_X + 1];
            for(int i=0; i<=MAX_INDEX_Y; i++)
            {
                for(int j=0; j<=MAX_INDEX_X; j++)
                {
                    if (blocks[i, j] == null) continue;

                    if(j<=MAX_INDEX_X-2
                    && blocks[i, j+1] != null && blocks[i, j+1].blockType == blocks[i, j].blockType
                    && blocks[i, j+2] != null && blocks[i, j+2].blockType == blocks[i, j].blockType)
                    {
                        destroyCheck[i, j] = true;
                        destroyCheck[i, j+1] = true;
                        destroyCheck[i, j+2] = true;
                    }
                    if (i <= MAX_INDEX_Y - 2
                    && blocks[i + 1, j] != null && blocks[i + 1, j].blockType == blocks[i, j].blockType
                    && blocks[i + 2, j] != null && blocks[i + 2, j].blockType == blocks[i, j].blockType)
                    {
                        destroyCheck[i, j] = true;
                        destroyCheck[i+1, j] = true;
                        destroyCheck[i+2, j] = true;
                    }
                }
            }
            bool checkPlaySound = false;
            for (int i = 0; i <= MAX_INDEX_Y; i++)
            {
                for (int j = 0; j <= MAX_INDEX_X; j++)
                {
                    if (destroyCheck[i, j] == true)
                    {
                        blocks[i, j] = null;
                        if(isSelect && selectY == i && selectX == j)
                        {
                            isSelect = false;
                        }
                        score++;
                        checkPlaySound = true;

                        if (score > highScore)
                            highScore = score;
                    }
                }
            }
            if(checkPlaySound == true)
                MainPage.gameEngine.PlaySound(CRes.sound1);
            #endregion

            #region 블록 아래가 비었으면 내리기
            for (int i = MAX_INDEX_Y; i >= 1; i--)
            {
                for(int j=0; j<=MAX_INDEX_X; j++)
                {
                    if (blocks[i, j] == null) continue;
                    if(blocks[i-1, j] == null)
                    {
                        blocks[i - 1, j] = blocks[i, j];
                        blocks[i - 1, j].y = GAME_HEIGHT - ((i-1) * BLOCK_HEIGHT) - BLOCK_HEIGHT/2;
                        blocks[i, j] = null;
                        if (isSelect && selectY == i && selectX == j)
                        {
                            isSelect = false;
                        }
                    }
                }
            }
            #endregion
        }

        public void MouseProcess() // 마우스 입력 처리
        {
            if(MainPage.gameEngine.MouseDownEvent == true)
            {
                if (isPlaying == false)
                {
                    InitGame();
                    isPlaying = true;
                }
                else
                {
                    Point p = MainPage.gameEngine.pointMouseDown;
                    
                    if(p.Y < GAME_HEIGHT)
                    {
                        int idxX, idxY;
                        idxX = (int)(p.X / BLOCK_WIDTH);
                        idxY = (int)((GAME_HEIGHT - p.Y) / BLOCK_HEIGHT);
                        if (isSelect == false)
                        {
                            if (blocks[idxY, idxX] != null)
                            {
                                selectX = idxX;
                                selectY = idxY;
                                isSelect = true;
                            }
                        }
                        else
                        {
                            if (blocks[idxY, idxX] != null)
                            {
                                BlockType temp;
                                temp = blocks[idxY, idxX].blockType;
                                blocks[idxY, idxX].blockType = blocks[selectY, selectX].blockType;
                                blocks[selectY, selectX].blockType = temp;
                                isSelect = false;
                            }
                            else
                            {
                                isSelect = false;
                            }
                        }
                    }
                }
                MainPage.gameEngine.MouseDownEvent = false;
            }
        }

        public void Render() // 드로잉 처리
        {
            if (isPlaying == false)
            {
                MainPage.gameEngine.DrawText("Click to start!", "굴림", Color.FromArgb(255, 255, 255, 255), 12, 110.0, 460.0);
            }
            //else
            {
                if (dropBlock != null)
                {
                    DrawBlock(dropBlock);
                }
                for(int i=0; i<=MAX_INDEX_Y; i++)
                {
                    for(int j=0; j<=MAX_INDEX_X; j++)
                    {
                        if (blocks[i, j] != null)
                        {
                            DrawBlock(blocks[i, j]);
                        }
                    }
                }

                if(isSelect)
                {
                    MainPage.gameEngine.DrawImage(CRes.blockSelect, selectX * BLOCK_WIDTH, GAME_HEIGHT - (selectY * BLOCK_HEIGHT) - BLOCK_HEIGHT);
                }

                DrawScore();
            }
        }

        public void DrawBlock(Block block)
        {
            switch(block.blockType)
            {
                case BlockType.Red:
                    MainPage.gameEngine.DrawImage(CRes.blockRed, block.x - BLOCK_WIDTH / 2, block.y - BLOCK_HEIGHT / 2);
                    break;
                case BlockType.Green:
                    MainPage.gameEngine.DrawImage(CRes.blockGreen, block.x - BLOCK_WIDTH / 2, block.y - BLOCK_HEIGHT / 2);
                    break;
                case BlockType.Blue:
                    MainPage.gameEngine.DrawImage(CRes.blockBlue, block.x - BLOCK_WIDTH / 2, block.y - BLOCK_HEIGHT / 2);
                    break;
                case BlockType.Yellow:
                    MainPage.gameEngine.DrawImage(CRes.blockYellow, block.x - BLOCK_WIDTH / 2, block.y - BLOCK_HEIGHT / 2);
                    break;
                case BlockType.Purple:
                    MainPage.gameEngine.DrawImage(CRes.blockPurple, block.x - BLOCK_WIDTH / 2, block.y - BLOCK_HEIGHT / 2);
                    break;
            }
        }

        public void DrawScore()
        {
            MainPage.gameEngine.DrawText("Score : " + score.ToString(), "굴림", Color.FromArgb(255, 255, 255, 255), 12, 10.0, 460.0);
            MainPage.gameEngine.DrawText("High: " + highScore.ToString(), "굴림", Color.FromArgb(255, 255, 255, 255), 12, 240.0, 460.0);
        }
    }
}